@@ -15,9 +15,9 @@ import Settings.Parser
 import System.Console.GetOpt
 import System.Environment
 import qualified System.Directory as Directory
+import Debug.Trace ( trace )
 
 import qualified Data.Set as Set
-import Data.Maybe
 
 data TestSpeed = TestSlow | TestNormal | TestFast deriving (Show, Eq)
 
@@ -114,7 +114,7 @@ data DocArgs = DocArgs
   } deriving (Eq, Show)
 
 defaultDocArgs :: DocArgs
-defaultDocArgs = DocArgs { docsBaseUrl = "../%pkg%" }
+defaultDocArgs = DocArgs { docsBaseUrl = "../%pkgid%" }
 
 readConfigure :: Either String (CommandLineArgs -> CommandLineArgs)
 readConfigure = Left "hadrian --configure has been deprecated (see #20167). Please run ./boot; ./configure manually"
@@ -139,7 +139,9 @@ readFreeze2 = Right $ \flags -> flags { freeze1 = True, freeze2 = True }
 readSkipDepends = Right $ \flags -> flags { skipDepends = True }
 
 readUnitIdHash :: Either String (CommandLineArgs -> CommandLineArgs)
-readUnitIdHash = Right $ \flags -> flags { unitIdHash = True }
+readUnitIdHash = Right $ \flags ->
+  trace "--hash-unit-ids is deprecated. It is enabled by release flavour or +hash_unit_ids flavour transformer" $
+  flags { unitIdHash = True }
 
 readProgressInfo :: String -> Either String (CommandLineArgs -> CommandLineArgs)
 readProgressInfo ms =
@@ -192,11 +194,11 @@ readTestOnlyPerf = Right $ \flags -> flags { testArgs = (testArgs flags) { testO
 readTestSkipPerf :: Either String (CommandLineArgs -> CommandLineArgs)
 readTestSkipPerf = Right $ \flags -> flags { testArgs = (testArgs flags) { testSkipPerf = True } }
 
-readHaddockBaseUrl :: Maybe String -> Either String (CommandLineArgs -> CommandLineArgs)
-readHaddockBaseUrl base_url = Right $ \flags ->
-  flags { docsArgs = (docsArgs flags) { docsBaseUrl = base_url' } }
+readHaddockBaseUrl :: Either String (CommandLineArgs -> CommandLineArgs)
+readHaddockBaseUrl = Right $ \flags ->
+  flags { docsArgs = (docsArgs flags) { docsBaseUrl = base_url } }
 
-  where base_url' = fromMaybe "https://hackage.haskell.org/package/%pkg%/docs" base_url
+  where base_url = "/package/%pkg%/docs"
 
 
 readTestRootDirs :: Maybe String -> Either String (CommandLineArgs -> CommandLineArgs)
@@ -279,7 +281,7 @@ optDescrs =
     , Option [] ["skip-depends"] (NoArg readSkipDepends)
       "Skip rebuilding dependency information."
     , Option [] ["bignum"] (OptArg readBignum "BACKEND")
-      "Select ghc-bignum backend: native, gmp (default), check-gmp, ffi."
+      "Select bignum backend: native, gmp (default), check-gmp (gmp compared to native), ffi."
     , Option [] ["progress-info"] (ReqArg readProgressInfo "STYLE")
       "Progress info style (None, Brief, Normal or Unicorn)."
     , Option [] ["docs"] (ReqArg readDocsArg "TARGET")
@@ -320,8 +322,8 @@ optDescrs =
         "Destination path for the bindist 'install' rule"
     , Option [] ["complete-setting"] (OptArg readCompleteStg "SETTING")
         "Setting key to autocomplete, for the 'autocomplete' target."
-    , Option [] ["haddock-base-url"] (OptArg readHaddockBaseUrl "BASE_URL")
-        "Generate documentation suitable for upload to hackage or for another base URL (for example a local hackage server)."
+    , Option [] ["haddock-for-hackage"] (NoArg readHaddockBaseUrl)
+        "Generate documentation suitable for upload to a hackage server."
     ]
 
 -- | A type-indexed map containing Hadrian command line arguments to be passed
