@@ -28,7 +28,7 @@ for working with type-level naturals should be defined in a separate library.
 
 module GHC.Internal.TypeNats
   ( -- * Nat Kind
-    Natural -- declared in GHC.Num.Natural in package ghc-bignum
+    Natural -- declared in GHC.Internal.Bignum.Natural
   , Nat
     -- * Linking type and value level
   , KnownNat(natSing), natVal, natVal'
@@ -40,7 +40,7 @@ module GHC.Internal.TypeNats
   , SNat (UnsafeSNat)
       -- We export a pattern synonym instead of the real constructor:
       -- See Note [Preventing unsafe coercions for singleton types].
-  , pattern SNat
+  , data SNat
   , fromSNat
   , withSomeSNat
   , withKnownNat
@@ -56,11 +56,11 @@ module GHC.Internal.TypeNats
 
 import GHC.Internal.Base( Eq(..), Functor(..), Ord(..), WithDict(..), (.), otherwise
                , Void, errorWithoutStackTrace, (++))
-import GHC.Types
-import GHC.Num.Natural(Natural)
+import GHC.Internal.Types
+import GHC.Internal.Bignum.Natural(Natural)
 import GHC.Internal.Show(Show(..), appPrec, appPrec1, showParen, showString)
 import GHC.Internal.Read(Read(..))
-import GHC.Prim(Proxy#)
+import GHC.Internal.Prim(Proxy#)
 import GHC.Internal.Data.Either(Either(..))
 import GHC.Internal.Data.Maybe(Maybe(..))
 import GHC.Internal.Data.Proxy (Proxy(..))
