@@ -22,12 +22,10 @@ import qualified Data.ByteString.Lazy.Char8 as BSL
 import System.Info hiding (compilerVersion)
 import System.Exit
 import qualified System.IO as IO
-import Options.Applicative
 
 -- Shake - build system
 import Development.Shake
 import Development.Shake.FilePath hiding (exe)
-import Development.Shake.Util as SU
 
 import RunnerTypes
 import qualified Measurements as Ms
@@ -37,9 +35,6 @@ import qualified CachegrindParse
 import qualified PerfStatParse
 
 import Utils
-import Debug.Trace
-import Control.DeepSeq
-import Control.Concurrent
 
 -- | A handy shortcut.
 ml :: String -> Label
@@ -48,13 +43,15 @@ ml = Ms.mkLabel
 ---------------------------------------------------------------------
 -- TEST CONFIGURATION - which tests are available to run
 
--- | These are directories that contain tests.
+-- | These are directories that we look into for tests by default.
 testRoots :: [String]
-testRoots = words "imaginary spectral real shootout parallel gc smp"
-
--- | These tests are run by default
-defaultRoots :: [String]
-defaultRoots = words "imaginary spectral real shootout"
+testRoots = words "imaginary spectral real shootout"
+            -- Note that we don't run the gc smp and parallel tests by default.
+            -- See #24 for some of the reasoning. The short version is that they are not very
+            -- stable run to run and are more sensitive to system load. So more care has to be
+            -- taken to properly run these.
+            -- Currently we value benchmark stability higher than including these in the default set
+            -- therefore we skip these by default.
 
 defaultNoFibHcOpts :: [String]
 defaultNoFibHcOpts = words "-O2 -Wno-tabs"
@@ -245,20 +242,23 @@ buildRules nofib@Build{..} = do
     -- .stamp file per build directory.
     buildDepsStamp %> \out -> do
         configs <- mapM (getTestConfig nofib) tests
-        let deps = nub $ foldMap (\config -> words $ config "SRC_DEPS") configs
+        let deps = nub $ ["base"] ++ foldMap (\config -> words $ config "SRC_DEPS") configs
         -- TODO: Invoking cabal in the way we do without any package argument fails.
         root <- liftIO $ IO.makeAbsolute buildDepsRoot
         unless (null deps) $ withResource pkgDbResource 1 $
-            cmd_ "cabal" ("--store-dir=" <> root) "v2-install" "--lib" "-w" compiler "--allow-newer" deps ("-j"<> show threads)
+            let project_file = if useHackageHead then "--project-file=cabal.project.head-hackage" else ""
+            in
+            cmd_ "cabal" project_file ("--store-dir=" <> root) "v2-install" "--lib" "-w" compiler "--allow-newer" deps ("-j"<> show threads)
         liftIO $ writeFile out ""
 
     -- Benchmark rules
     "_make//config.txt" %> \out -> do
         let test = unoutput out
         let src_dir = testDir test
-        printM $ "config:out:" ++ out
-        printM $ "config:test:" ++ show test
-        printM $ "config:src_dir:" ++ src_dir
+        runVerbose nofib $ do
+            printM $ "config:out:" ++ out
+            printM $ "config:test:" ++ show test
+            printM $ "config:src_dir:" ++ src_dir
         src <- lines . fuseLines <$> readFile' (src_dir </> "Makefile")
         writeFileLines out $ convertConfig src
 
@@ -280,14 +280,15 @@ buildRules nofib@Build{..} = do
                 -- Not recursive, the make system ignores files in subfolders.
                 c_srcs <- getDirectoryFiles "" [src_dir ++ "/*.c"]
                 need c_srcs
-                printM $ "c_srcs" ++ show c_srcs
+                runVerbose nofib $ printM $ "c_srcs" ++ show c_srcs
                 let c_objects = map (\c_file -> replaceExtension (output </> c_file) "o") c_srcs
                 return $ nub $ hs_objects ++ c_objects
               else do
                 map (obj_dir </>) <$> mapM oFromSrc config_srcs
 
-        liftIO $ print "OBJS"
-        liftIO $ mapM_ putStrLn objs
+        liftIO $ runVerbose nofib $ do
+            print "OBJS"
+            mapM_ putStrLn objs
 
         need $ nub objs
 
@@ -327,20 +328,24 @@ buildRules nofib@Build{..} = do
             src_dir = testDir test          -- eg. spectral/simple
             obj_dir = output </> src_dir    -- eg. _make/foo/spectral/simple
             o_name = drop (length obj_dir + 1) o -- eg. Main.o
-        printM "readConfig"
 
-        printM $ "o:" ++ o
-        printM $ "test:" ++ show test
-        printM $ "obj_dir:" ++ obj_dir
-        printM $ "src_dir:" ++ src_dir
-        printM $ "o_name:" ++ o_name
+        runVerbose nofib $ do
+            printM "readConfig"
+
+            printM $ "o:" ++ o
+            printM $ "test:" ++ show test
+            printM $ "obj_dir:" ++ obj_dir
+            printM $ "src_dir:" ++ src_dir
+            printM $ "o_name:" ++ o_name
 
         -- Deps are needed for build dependencies/src files
         -- But we only care about lines starting with the current .o file
         deps <- filter (\x -> head (words x) `equalFilePath` o) <$>
                     readFileLines (obj_dir </> ".depends")
-        printM $ "deps:"
-        printM $ deps
+
+        runVerbose nofib $ do
+            printM $ "deps:"
+            printM $ deps
 
         -- Figure out source location
         -- Haskell sources are recorded in .depends so look there first.
@@ -358,14 +363,15 @@ buildRules nofib@Build{..} = do
         -- file in the future for example.
         need [src]
 
-        printM $ "src1:" ++ src
+        runVerbose nofib $ do
+            printM $ "src1:" ++ src
 
-        printM $ "mod_name:" ++ o_name
-        printM $ "src:" ++ src
+            printM $ "mod_name:" ++ o_name
+            printM $ "src:" ++ src
 
         -- foo.o : foo.hs => foo.hs
         let needs = [ drop 2 . dropWhile (/= ':') $ x | x <- deps]
-        liftIO $ do
+        runVerbose nofib $ liftIO $ do
             print "needs"
             mapM_ print needs
         need needs
@@ -446,13 +452,16 @@ buildRules nofib@Build{..} = do
 
     -- Run tests under cachegrind
     ["//Main.cachegrind.results.tsv"] &%> \[resultsTsv] -> do
-        out' <- liftIO $ IO.canonicalizePath resultsTsv
         let test = testFromResultTsv nofib resultsTsv
+            src_dir = testDir test          -- eg. spectral/simple
+            obj_dir = output </> src_dir    -- eg. _make/foo/spectral/simple
+        out' <- liftIO $ IO.canonicalizePath resultsTsv
+        obj_dir' <- liftIO $ IO.canonicalizePath obj_dir
         let cachegrindOut n = FP.replaceFileName out' ("Main.cachegrind.result" <.> show n)
         let wrapper_args n =
               ["valgrind", "--tool=cachegrind"] <> cachegrind_args <>
               [ "--cachegrind-out-file="<>cachegrindOut n
-              , "--log-file=cachegrind.log"
+              , "--log-file=" <> obj_dir' </> "cachegrind.log"
               ]
 
         let parse_cachegrind n = do
@@ -468,6 +477,7 @@ data RunMode  = ModeRun -- ^ Regular runtime measurement
               -- | Wrap the executable by a call to another executable.
               --
               -- For example perf or valgrind.
+              -- The int is the n'th run
               | ModeWrapped (Int -> [String])
                             (Int -> IO (Measurements Double))
 
@@ -551,7 +561,7 @@ runTest nofib@Build{..} runMode resultsTsv = do
                             putStrLn "Start of expected output:"
                             BSL.putStrLn $ BSL.take 800 stdout'
                             putStrLn $ "Full expected/actual output written to " ++ (output </> src_dir)
-                        fail $ "Benchmark failed:" ++ src_dir ++ " unexpected stdout:^"
+                        unless keepGoing $ fail $ "Benchmark failed:" ++ src_dir ++ " unexpected stdout:^"
                   -- No stdout file, ignore stdout
                   _ -> return ()
 
@@ -601,7 +611,7 @@ getTestCompileArgs nofib test = do
     config <- getTestConfig nofib test
     return $ defaultNoFibHcOpts
          ++ words (config "SRC_HC_OPTS")
-         ++ [ "-package-env", "-" ]
+         ++ [ "-package-env", "-", "-package", "base"]
          ++ concat [ ["-package", pkg] | pkg <- words (config "SRC_DEPS") ]
 
 getModeArgs :: (String -> String) -> Speed -> [String]
@@ -620,7 +630,7 @@ getModeArgs benchSettings speed = words $
 
 -- | Get stdin, arguments, expected stdout for benchmarks
 getTestCmdline :: Nofib -> TestName -> Action (BSL.ByteString, [String], Maybe (BSL.ByteString), Maybe (BSL.ByteString))
-getTestCmdline Build{..} test = do
+getTestCmdline nofib@Build{..} test = do
     config <- readConfig' $ output </> src_dir </> "config.txt"
 
     -- Mode/Speed args default to normal mode.
@@ -636,7 +646,7 @@ getTestCmdline Build{..} test = do
       in if s == ""
             then liftIO $ grab "stdin"
             else pure $ Just $ testDir test </> s
-    liftIO $ putStrLn $ "test " <> unTestName test <> " stdin: " <> show stdin_path
+    liftIO $ runVerbose nofib $ putStrLn $ "test " <> unTestName test <> " stdin: " <> show stdin_path
     stdin <- liftIO $ maybe (pure BSL.empty) BSL.readFile stdin_path
 
     -- Check if there is an stdout file. Default to Nothing
@@ -645,7 +655,7 @@ getTestCmdline Build{..} test = do
       in if null s
             then liftIO $ grab "stdout"
             else pure $ Just $ testDir test </> s :: Action (Maybe FilePath)
-    liftIO $ putStrLn $ "test " <> unTestName test <> " stdout: " <> show stdout_path
+    liftIO $ runVerbose nofib $ putStrLn $ "test " <> unTestName test <> " stdout: " <> show stdout_path
     stdout <- liftIO $ sequence (BSL.readFile <$> stdout_path)
 
     -- Check if there is an stdout file. Default to Nothing
@@ -654,7 +664,7 @@ getTestCmdline Build{..} test = do
       in if null s
             then liftIO $ grab "stderr"
             else pure $ Just $ testDir test </> s :: Action (Maybe FilePath)
-    liftIO $ putStrLn $ "test " <> unTestName test <> " stderr: " <> show stderr_path
+    liftIO $ runVerbose nofib $ putStrLn $ "test " <> unTestName test <> " stderr: " <> show stderr_path
     stderr <- liftIO $ sequence (BSL.readFile <$> stderr_path)
 
     return (stdin, args, stdout, stderr)
@@ -710,8 +720,7 @@ convertConfig xs =
 readConfig :: FilePath -> IO (String -> String)
 readConfig x = do
     -- printM $ "readConfig:" ++ x
-    src <- fuseLines <$> readFile x
-    -- !_ <- return $ length src
+    src <- fuseLines <$> IO.readFile' x
     let res = [ (reverse $ dropWhile isSpace $ reverse a, dropWhile isSpace $ drop 1 b)
               | y <- lines src
               , let (a,b) = break (== '=') y
